
/**
 * Copyright (c) Milad Eftekhar <milad@cs.toronto.edu>
 * @author Milad
 *
 */


import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;


public class FGC {
	
	public static float influentialgroup(int l, int budget, String graphfile, String groupfile, final int BETA, final int R) throws IOException{ 	
		
		/*
		 * input: 
		 * l = the number of wanted influential groups
		 * budget = the advertising budget (k)
		 * graphfile = the filename containing the social graph.
		 * groupfile = the filename containing groups.
		 * 
		 * output:
		 * print: the l most influential groups
		 * return: the average cascade value (the number of convinced nodes) when the seedset is the printed nodes (the l most influential nodes) and the experiment is simulated for R different delta settings.
		 *
		 */
		
		//printing parameter setting
		System.out.println("l: "+l);
		System.out.println("budget: "+budget);
		System.out.println("graphfile: "+graphfile);
		System.out.println("groupfile: "+groupfile);
		
		

		
		//Reading the social graph
		ArrayList<weightedEdge>[] socialgraph = readFile.readEdges_from_file(graphfile, new Hashtable<HTNode, Integer>());	

		//Reading the saved R settings for delta
		System.out.println("Thresholds fetched! R: "+ R);
		
		ArrayList<Integer>[] groups = readFile.readMClusters_from_file(groupfile);
		int g_dimension = groups.length;
		
		//Initialization
		//int[] BETAS = {10,20,30,40,50,60,70,80,90,100};
		boolean[] used = new boolean[g_dimension];
		int[] InfNodes = new int[l];
		
		float group_budget = (float) BETA*budget/l;
		
		//Measuring run time
		long start = new Date().getTime();
			
		float maxcascade = 0;
		
		for(int i = 0; i<l; i++){
					
			System.out.println("Start to explore for "+i+"th influnetial group");
			maxcascade=-1; int maxnode=-1;
			for(int j=0; j<g_dimension; j++){
				if (!used[j]){
					InfNodes[i] = j;
					
					float sum_sim=0;	//saves the sum of val in R simulations (different delta settings).
					for(int phase=0; phase<R; phase++){
						sum_sim += compute_Revenue(InfNodes,i,group_budget,socialgraph,groups);
					}
					sum_sim/=R;
					
					if (sum_sim>maxcascade){
						maxcascade = sum_sim;
						maxnode = j;
					}
				}
			}
			
			InfNodes[i] = maxnode;
			used[maxnode]=true;
			
			System.out.println(i+"th influential group is: "+maxnode);
			System.out.println("Cascade is: "+maxcascade);
			
		}
		
		long finish = new Date().getTime();
		
		//Printing the run time
		System.out.println("The run time (ms) is: "+(finish-start));
		
		//Printing k Influential Nodes
		System.out.println(l+" Influential groups:");
		for(int infnode : InfNodes)
			System.out.print(infnode+" ");
		System.out.println();
		
		//returns the average cascade value (the number of convinced nodes) of seedset InfNodes for R different delta settings.
		return maxcascade;		
	}
		
	

	public static int compute_Revenue(int[] seedset, int index, float group_budget, ArrayList<weightedEdge>[] socialgraph, ArrayList<Integer>[] groups){
		
		
		int dimension = socialgraph.length;
		
		float[] delta = new float[dimension];
		for (int i = 0; i < dimension; i++)
			delta[i] = (float) Math.random();
		
		float[] init_inf = new float[dimension];
				
		for(int i=0; i<=index; i++){
			int groupSize = groups[seedset[i]].size();
			float budget =(float) group_budget/groupSize;	
			
			for(int person : groups[seedset[i]])
				init_inf[person] += budget;
				
		}

			
		return IC.compute(init_inf, socialgraph, delta);
	}
	
	public static void main(String[] args) throws IOException {
		//args[0]: l the number of desired groups
		//args[1]: advertising budget (k)
		//args[2]: the filename containing the social graph
		//args[3]: the filename containing groups
		//args[4]: BETA
		//args[5]: R
		
		System.out.println(influentialgroup(Integer.parseInt(args[0]),Integer.parseInt(args[1]),args[2],args[3],Integer.parseInt(args[4]),Integer.parseInt(args[5])));

	}

}
