
/**
 * Copyright (c) Milad Eftekhar <milad@cs.toronto.edu>
 * @author Milad
 *
 */

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Hashtable;
import java.util.LinkedList;


public class baseline {

	public static int[] random_groups(int l, int g_dimension){
		int[] InfNodes = new int[l];
		for(int i=0; i<l; i++){
			boolean usable = false;
			int rand = -1;
			while(!usable){
				rand = (int) (Math.random()*g_dimension);
				usable = true;
				for(int j=0; j<i; j++){
					if(InfNodes[j] == rand){
						usable = false;
						break;
					}
				}
			}			
			InfNodes[i] = rand;
		}
		
		return InfNodes;

	}
	
	public static int[] big_groups(int l, ArrayList<Integer>[] groups){
		int g_dimension = groups.length;
		Node[] size = new Node[g_dimension];
		for (int i = 0; i< g_dimension; i++)
			size[i] = new Node(i,groups[i].size());
		Arrays.sort(size);
		
		int[] InfNodes = new int[l];
		for(int i=0; i<l; i++){
			InfNodes[i] = size[i].getind();
			System.out.print(size[i].getind()+":"+size[i].getval()+" , ");
		}
		return InfNodes;
	}
	
	public static int[] small_groups(int l, ArrayList<Integer>[] groups){
		int g_dimension = groups.length;
		Node[] size = new Node[g_dimension];
		for (int i = 0; i< g_dimension; i++)
			size[i] = new Node(i,groups[i].size());
		Arrays.sort(size);
		
		int[] InfNodes = new int[l];
		
		int index = 0;
		while (size[g_dimension-index-1].getval()==0)
			index++;
		
		
		for(int i=0; i<l; i++){
			InfNodes[i] = size[g_dimension-index-i-1].getind();
			System.out.print(size[g_dimension-index-i-1].getind()+":"+size[g_dimension-index-i-1].getval()+" , ");
		}
		return InfNodes;
		
	}
	
	public static int[] high_degree_groups(int l, ArrayList<weightedEdge>[] socialgraph,ArrayList<Integer>[] groups,ArrayList<Integer>[] memberships){
		int g_dimension = groups.length;
		float[][] groupmodel = CGC.modeling(socialgraph, groups, memberships);
		float[] weight = new float[g_dimension];
		for(int i=0; i<g_dimension; i++)
			for(float f : groupmodel[i])
				weight[i] += f;
		
		Node[] totalweight = new Node[g_dimension];
		for(int i=0; i<g_dimension; i++)
			totalweight[i] = new Node(i,weight[i]);
		
		Arrays.sort(totalweight);

		int[] InfNodes = new int[l];
		for(int i=0; i<l; i++){
			InfNodes[i] = totalweight[i].getind();
			System.out.print(totalweight[i].getind()+":"+totalweight[i].getval()+" , ");
		}
		return InfNodes;
		

	}

	
public static int computeFaster(int[] seedset, int index, float[][] groupgraph){
		
		/* Applicable when init_inf all members in seedset is 1 that happens when we run the individual case IC. 
		 * When we call IC for other algorithms like FGC and CGC we cannot use this function since init_inf is not 1 (it is a number between 0 and 1); 
		 * hence we need to use compute(...) in these cases.
		 * 
		 * input:
		 * seedset: the seed set
		 * index: the index of last member in seedset = |seedset|-1
		 * init_inf: the influenced value for the seedset members.
		 * socialgraph
		 * delta: showing the threshold for each node in the graph
		 * dimension: size of the social graph
		 * 
		 * output:
		 * return: the cascade (number of convinced nodes) as the result of convincing seed set Nodes. 
		 */
		
		int cascade = 0;
		int dimension = groupgraph.length;
		
		float[] delta = new float[dimension];
		for (int i=0; i<dimension; i++)
			delta[i] = (float) Math.random();
		
		float[] init_inf = new float[dimension];

		//Initializing the active nodes.
		LinkedList<Integer> active = new LinkedList<Integer>();
		for(int i=0; i<= index; i++){
				active.add(seedset[i]);
				init_inf[seedset[i]]=1;
				++cascade;
		}
		
		//Simulation.
		Integer current;
		while((current=active.poll())!=null){
			
			for(int neighbor=0; neighbor<dimension; neighbor++){
				if(neighbor!=current){
					float newval = init_inf[neighbor]+groupgraph[current][neighbor];
					if(newval>=delta[neighbor] && init_inf[neighbor]<delta[neighbor]){
						active.add(neighbor);
						init_inf[neighbor]=1;
						++cascade;
					}
					if(newval<delta[neighbor])
						init_inf[neighbor]=newval;
				}
			}
		}
		

		return cascade;
	}
	
	public static void main(String[] args) throws IOException{

/*		
  		args[0]: l the number of desired groups
		args[1]: advertising budget (k)
		args[2]: the filename containing the social graph
		args[3]: the filename containing groups (group -> a list of users): dblp_MClusters.bin
		args[4]: the filename containing memberships (user -> a list of groups): dblp_groups.bin
		args[5]: PROB
		args[6]: BETA
		args[7]: R
		args[8]: algNum (which algorithm: 1:random 2:big 3:small 4:degree)
		
*/
		
		//printing parameter setting
		int l = Integer.parseInt(args[0]);
		int budget = Integer.parseInt(args[1]);
		int R = Integer.parseInt(args[7]);
		int algNum = Integer.parseInt(args[8]);
		
		System.out.println("l: "+l);
		System.out.println("budget: "+budget);
		System.out.println("graphfile: "+args[2]);
		System.out.println("groupfile: "+args[3]);
		System.out.println("membershipfile: "+args[4]);
		
		//Reading the social graph
		ArrayList<weightedEdge>[] socialgraph = readFile.readEdges_from_file(args[2], new Hashtable<HTNode, Integer>());	

		//Reading the MClusters (a group -> a list of users belonging to that group) 
		ArrayList<Integer>[] groups = readFile.readMClusters_from_file(args[3]);
		int g_dimension = groups.length;
		
		//Reading the group memberships ( a user -> a list of groups this user belongs to)
		ArrayList<Integer>[] memberships = readFile.readMClusters_from_file(args[4]);
		
		float group_budget = (float) Integer.parseInt(args[6])*budget/l;
		
		long start = new Date().getTime();
		
		int[] InfNodes = new int[l];
		double cascade = 0;

		switch (algNum){
			case 1:
				for(int i=0; i<10; i++){
					InfNodes = random_groups(l,g_dimension);
					for(int phase = 0; phase < R; phase++)
						cascade += FGC.compute_Revenue(InfNodes, l-1, group_budget, socialgraph, groups);
				}
				cascade /= R*10;
				break;
				
			case 2:
				InfNodes = big_groups(l, groups);
				for(int phase = 0; phase < R; phase++)
					cascade += FGC.compute_Revenue(InfNodes, l-1, group_budget, socialgraph, groups);		
				cascade /= R;
				break;
				
			case 3:
				InfNodes = small_groups(l, groups);
				for(int phase = 0; phase < R; phase++)
					cascade += FGC.compute_Revenue(InfNodes, l-1, group_budget, socialgraph, groups);		
				cascade /= R;
				break;
				
			case 4:
				InfNodes = high_degree_groups(l, socialgraph, groups, memberships);
				for(int phase = 0; phase < R; phase++)
					cascade += FGC.compute_Revenue(InfNodes, l-1, group_budget, socialgraph, groups);		
				cascade /= R;				
				break;
				
		}

		long end = new Date().getTime();
		
		//returns the average cascade value (the number of convinced nodes) of seedset InfNodes for R different delta settings.
		//This is a simulation of linear threshold models consistent with results of IC and FGC to be comparable.
		
		
		System.out.println("case "+algNum+" time (ms): "+(end-start));
		System.out.println("case "+algNum+" cascade: "+cascade);



	}

}
